/**
Beschreibung der Klasse Patient
 
- author: Klaus Reinold
- version: 1.0
*/
class Patient
{
    /** Name des Patienten */
    private var name: String

    /**
    Konstruktor des Patienten
    - parameters:
        - s: Name des Patienten
    */
    init(s: String)
    {
        name = s
    }

    /**
    Gibt Information über den Patienten aus.
    */
    func InformationAusgeben()
    {
        print(name)
    }

    /**
    Gibt den Namen des Patienten aus
    - returns: Name des Patienten
    */
    func NameGeben() -> String
    {
        return name
    }
    
    /**
    Gibt zurück, ob der Name gleich den übergebenen Wert ist.
    - parameters:
        - gesuchterWert: gesuchter Wert
    - returns: Wahrheitswert, der angibt, ob der Name der gesuchte ist.
    */
    func SchlüsselIstGleich(gesuchterWert: String) -> Bool
    {
        return name == gesuchterWert
    }
    
    /**
    Vergleich zweier Datenelemente bezüglich einer Ordnungsrelation
    - parameters:
        - dvergleich: Datenelement zum Vergleich
    */
    func IstKleinerAls(dvergleich: Patient) -> Bool
    {
        return name < dvergleich.name
    }
}

/**
Beschreibung der Klasse Knoten

- author: Klaus Reinold
- version: 1.0
*/
class Knoten
{
    /** Verwaltung des nachfolgenden Knoten in der Liste. */
    private var nachfolger: Knoten?
    /** Verwaltung der Daten des Patienten */
    private var daten: Patient

    /**
    Konstruktor des Knoten
    - parameters:
        - dNeu: Datenelement
    */
    init(dneu: Patient)
    {
        nachfolger  = nil
        daten = dneu
    }

    /**
    Konstruktor des Knoten
    - parameters:
        - dNeu: Datenelement
        - kNeu: neuer Nachfolger
    */
    init(dneu: Patient, kNeu: Knoten?)
    {
        nachfolger  = kNeu
        daten = dneu
    }

    /**
    Fügt ein Datenelement am Ende der Liste ein
    - parameters:
        - dneu: neues Datenelement
    */
    func HintenEinfügen(dneu: Patient)
    {
        if nachfolger != nil
        {
            nachfolger!.HintenEinfügen(dneu: dneu)
        }
        else
        {
            let kneu = Knoten(dneu: dneu)
            nachfolger = kneu
        }
    }

    /**
    Fügt ein Datenelement vor einem bestimmten anderen Datenelement ein
    - parameters:
        - dneu: neues Datenelement
        - d_vergleich: gesuchtes Datenelement, vor dem eingefügt werden soll
    - returns: Weitergabe des künftigen Nachfolgers
    */
    func EinfügenVor(dneu: Patient, d_vergleich: Patient) -> Knoten
    {
        if daten !== d_vergleich
        {
            if nachfolger != nil
            {
                nachfolger = nachfolger!.EinfügenVor(dneu: dneu, d_vergleich: d_vergleich)
            }
            else
            {
                HintenEinfügen(dneu: dneu)
            }
            return self
        }
        else
        {
            return Knoten(dneu: dneu, kNeu: self)
        }
    }
    
    /**
    Fügt ein Datenelement sortiert in die Liste ein
    - parameters:
        - dneu: neues Datenelement
    - returns: Weitergabe des künftigen Nachfolgers
    */
    func SortiertEinfügen(dneu: Patient) -> Knoten
    {
        if daten.IstKleinerAls(dvergleich: dneu)
        {
            if nachfolger != nil
            {
                nachfolger = nachfolger!.SortiertEinfügen(dneu: dneu)
            }
            else
            {
                HintenEinfügen(dneu: dneu)
            }
            return self
        }
        else
        {
            return Knoten(dneu: dneu, kNeu: self)
        }
    }
    
    /**
    Sucht ein Datenelement mit einem bestimmten Schlüssel in der Liste
    - parameters:
        - gesuchterWert: Zeichenkette, die als Schlüssel gefunden werden soll.
    - returns: das gesuchte Datenelement
    */
    func Suchen(gesuchterWert: String) -> Patient?
    {
        if daten.SchlüsselIstGleich(gesuchterWert: gesuchterWert)
        {
            return daten
        }
        else
        {
            if nachfolger != nil
            {
                return nachfolger!.Suchen(gesuchterWert: gesuchterWert)
            }
            else
            {
                return nil
            }
        }
    }

    /**
    Bestimmt die Länge der restlichen Liste ab dem aktuellen Knoten
    - returns: Länge der restlichen Liste
     ´*/
    func RestlängeGeben() -> Int
    {
        if nachfolger == nil
        {
            return 1
        }
        else
        {
            return nachfolger!.RestlängeGeben() + 1
        }
    }

    /**
    Nachfolger des Patienten setzen
    - parameters:
        - nf: neuer Nachfolger
    */
    func NachfolgerSetzen(nf: Knoten)
    {
        nachfolger = nf
    }

    /**
    Gibt den Nachfolger des Knoten aus.
    - returns: Nachfolger
    */
    func NachfolgerGeben() -> Knoten?
    {
        return nachfolger
    }
    
    /**
    Gibt das Datenelement zurück
    - returns: Datenelement
    */
    func DatenelementGeben() -> Patient
    {
        return daten
    }
    
    /**
    Gibt Information über den Patienten aus.
    */
    func InformationAusgeben()
    {
        daten.InformationAusgeben()
        if nachfolger != nil
        {
            nachfolger!.InformationAusgeben()
        }
    }

    /**
    Entfernt einen Knoten, der über sein Datenelement identifiziert wird, aus der Liste
    - parameters:
        - dvergleich: gesuchtes Datenelement
    - returns: Weitergabe des künftigen Nachfolgers
    */
    func KnotenEntfernen(dvergleich: Patient) -> Knoten?
    {
        if daten === dvergleich
        {
            return nachfolger
        }
        else
        {
            if nachfolger != nil
            {
                nachfolger=nachfolger!.KnotenEntfernen(dvergleich: dvergleich);
                return self
            }
            else
            {
                return nil
            }
        }
    }
    
    /**
    Gibt das Datenelement des letzten Knotens aus
    - returns: hinterstes Datenelement
    */
    func EndeGeben() -> Patient
    {
        if nachfolger == nil
        {
            return daten
        }
        else
        {
            return nachfolger!.EndeGeben()
        }
    }

    /**
    Gibt den letzten Knoten aus und entfernt ihn aus der Liste
    - returns: bisheriger letzter Knoten
    */
    func EndeEntfernen() -> Knoten?
    {
        if nachfolger == nil
        {
            return nil
        }
        else
        {
            nachfolger = nachfolger!.EndeEntfernen()
            return self
        }
    }
}

/**
Beschreibung der Liste

- author: Klaus Reinold
- version: 1.0
*/
class Liste
{
    /** erster Knoten der Liste */
    private var anfang: Knoten?

    /**
    Konstruktor der Liste
    */
    init()
    {
        anfang = nil
    }

    /**
     Fügt einen neuen Patienten am Ende der Liste ein.
    - parameters:
        - dNeu: neues Datenelement
    */
    func HintenEinfügen(dneu: Patient)
    {
        if anfang != nil
        {
            anfang!.HintenEinfügen(dneu: dneu)
        }
        else
        {
            anfang = Knoten(dneu: dneu)
        }
    }

    /**
    Fügt ein neues Datenelement am Anfang der Liste ein.
    - parameters:
        - dneu: neues Datenelement
    */
    func VorneEinfügen(dneu: Patient)
    {
        let kneu = Knoten(dneu: dneu, kNeu: anfang)
        anfang = kneu
    }

    /**
    Fügt ein neues Datenelement vor einem bestimmten Datenelement ein.
    - parameters:
        - dneu: neues Datenelement
        - d_vergleich: Datenelement, vor dem eingefügt werden soll
    */
    func EinfügenVor(dneu: Patient, d_vergleich: Patient)
    {
        if anfang != nil
        {
            anfang = anfang!.EinfügenVor(dneu: dneu, d_vergleich: d_vergleich)
        }
        else
        {
            VorneEinfügen(dneu: dneu)
        }
    }

    /**
    Fügt ein neues Datenelement sortiert in die Liste ein.
    - parameters:
        - dneu: neues Datenelement
    */
    func SortiertEinfügen(dneu: Patient)
    {
        if anfang != nil
        {
            anfang = anfang!.SortiertEinfügen(dneu: dneu)
        }
        else
        {
            VorneEinfügen(dneu: dneu)
        }
    }

    /**
    Bestimmt die Länge der Liste
    - returns: Listenlänge
    */
    func LängeGeben() -> Int
    {
        if anfang == nil
        {
            return 0
        }
        else
        {
            return anfang!.RestlängeGeben()
        }
    }

    /**
    Sucht ein Datenelement mit einem bestimmten Schlüssel in der Liste
    - parameters:
        - gesuchterWert: Zeichenkette, die als Schlüssel gefunden werden soll.
    - returns: das gesuchte Datenelement
    */
    func Suchen(gesuchterWert: String) -> Patient?
    {
        if anfang != nil
        {
            return anfang!.Suchen(gesuchterWert: gesuchterWert)
        }
        else
        {
            return nil
        }
    }

    /**
    Gibt Information über die Liste aus.
    */
    func InformationAusgeben()
    {
        if anfang != nil
        {
            anfang!.InformationAusgeben()
        }
    }

    /**
    Entfernt einen Knoten mit einem bestimmten Datenelement aus der Liste
    - parameters:
        - dvergleich: gesuchtes Datenelement
    */
    func KnotenEntfernen(dvergleich: Patient)
    {
        if anfang != nil
        {
            anfang = anfang!.KnotenEntfernen(dvergleich: dvergleich)
        }
    }

    /**
    Entfernt den ersten Knoten aus der Liste und gibt sein Datenelement aus.
    - returns: Datenelement des bisher ersten Knotens
    */
    func AnfangEntfernen() -> Patient?
    {
        if anfang != nil
        {
            let d = anfang!.DatenelementGeben()
            anfang = anfang!.NachfolgerGeben()
            return d
        }
        else
        {
            return nil
        }
    }

    /**
    Entfernt den letzten Knoten aus der Liste und gibt sein Datenelement aus.
    - returns: Datenelement des bisher letzten Knotens
    */
    func EndeEntfernen() -> Patient?
    {
        if anfang != nil
        {
            let d = anfang!.EndeGeben()
            anfang = anfang!.EndeEntfernen()
            return d
        }
        else
        {
            return nil
        }
    }
}
